/*---------------------------------------------------------------------------*/
/*                                PRG-2051                                   */
/*                                                                           */
/* Device programmer for the Atmel 89C2051 and 89C1051 microcontrollers.     */
/*                                                                           */
/* Filename:    SUBRS.C                                                      */
/* Description: Subroutines.                                                 */
/*                                                                           */
/* Copyright (c) 1995,1996,1997 Electronic Design Associates.                */
/* All rights reserved.                                                      */
/*                                                                           */
/*---------------------------------------------------------------------------*/

#include <prg2051.h>
#include <hardware.h>
#include <reg52.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <command.h>
#include <bios.h>
#include <main.h>


/*---------------------------------------------------------------------------*/
/*                            ClearBuffer()                                  */
/*---------------------------------------------------------------------------*/
void clear_buffer(void)
{
int index;

  for (index=0; index<BUFFER_SIZE; index++) {
    prog_buffer[index]=0;
  }

}


/*---------------------------------------------------------------------------*/
/*                            delay10ms()                                    */
/*---------------------------------------------------------------------------*/
void delay10ms(void)
{
  mstimer = 4*10;
  while(mstimer){};
}


/*---------------------------------------------------------------------------*/
/*                            delay10us()                                    */
/*---------------------------------------------------------------------------*/
void delay10us(void)
{
unsigned int delay;
  
  for (delay=10; delay; delay--) {}
}


/*---------------------------------------------------------------------------*/
/*                            device_on()                                    */
/*---------------------------------------------------------------------------*/
void device_on(void)
{
  output_enable();
  vcc_on();
  reset_off();
  clock_off();
  latch_disable();
  set_cmd(FLOAT);
  delay10ms();
  reset_on();
  prog_on();
}


/*---------------------------------------------------------------------------*/
/*                            device_off()                                   */
/*---------------------------------------------------------------------------*/
void device_off(void)
{
  clock_off();
  reset_off();
  latch_disable();
  output_disable();
  set_cmd(FLOAT);
  prog_on();
  vpp_off();
  vcc_off();
}


/*---------------------------------------------------------------------------*/
/*                              hex2bin()                                    */
/*---------------------------------------------------------------------------*/
unsigned char Hex[2];

unsigned char hex2bin(void)
{
unsigned char total;

  if (Hex[0]>='A')
    total  = (Hex[0]-(unsigned char)'A'+(unsigned char)10);
  else
    total  = (Hex[0]-(unsigned char)'0');
    
  if (Hex[1]>='A')
    total += (Hex[1]-(unsigned char)'A'+(unsigned char)10)*16;
  else
    total += (Hex[1]-(unsigned char)'0')*16;

  return(total);
}


/*---------------------------------------------------------------------------*/
/*                             prog_byte()                                   */
/*---------------------------------------------------------------------------*/
char prog_byte(char byte)
{
int timeout;

  set_cmd(WRITE_DATA);
  set_data(byte);
  latch_enable();
  vpp_on();
  delay10us();
  prog_off();
  delay10us();
  prog_on();
  delay10us();
  vpp_off();
  latch_disable();

  set_cmd(READ_DATA);
  for (timeout=300; timeout; timeout--) {
    if ( get_data() == byte ) return(0);
  }

  return(1);
}


/*---------------------------------------------------------------------------*/
/*                             strtoi()
/*---------------------------------------------------------------------------*/
int strtoi(char *cptr)
{
char n=0,count=0;
int i=0;

  while (isspace(*cptr)) cptr++;

  for(;;) {
    n=*cptr;
    if (isdigit(*cptr))
      n-='0';
    else if (islower(*cptr))
      n-='a'-10;
    else if (isupper(*cptr))
      n-='A'-10;
    else
      break;
    i = i*16 + n;
    cptr++;
    count++;
  }
  if (count)
    return(i);
  else 
    return(-1);
}



