/*---------------------------------------------------------------------------*/
/*                                PRG-2051                                   */
/*                                                                           */
/* Device programmer for the Atmel 89C2051 and 89C1051 microcontrollers.     */
/*                                                                           */
/* Filename:    UART.C                                                       */
/* Description: Serial port service routines.                                */
/*                                                                           */
/* Copyright (c) 1995,1996,1997 Electronic Design Associates.                */
/* All rights reserved.                                                      */
/*                                                                           */
/*---------------------------------------------------------------------------*/

#include <prg2051.h>
#include <hardware.h>
#include <reg52.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <subrs.h>
#include <command.h>
#include <bios.h>

#define REC_BUFFER_SIZE 50

/* globals */
char rec_buf[REC_BUFFER_SIZE];
data char rlen,*rptr;
data char rec_addr;
bit message=0;

/*---------------------------------------------------------------------------*/
/*                                keyboard()                                 */
/*                                                                           */
/* Process keyboard entries.                                                 */
/*---------------------------------------------------------------------------*/
void keyboard(void)
{
register char c;

  if (RI) {             /* serial receive */
    c=SBUF; 
    if (!message) {
      *rptr=c;
      switch(c) {
        case '\r':
          message=1;
          if (rptr<rec_buf+REC_BUFFER_SIZE)
            rptr++;
          else
            rptr=rec_buf+REC_BUFFER_SIZE;
          *rptr=NULL;
          break;
        case '\b':
          if (rlen) {
            rlen--;
            rptr--;
            SBUF=c;
            }
          break;
        default:
          if (rlen<REC_BUFFER_SIZE-1) {
            rptr++;
            rlen++;
            SBUF=c;
           }
        }
    }
    RI = 0;
  }
   
}


/*---------------------------------------------------------------------------*/
/*                                 do_comm()                                 */
/*                                                                           */
/* Process serial messages.                                                  */
/*---------------------------------------------------------------------------*/
void do_comm(void)
{
unsigned long int temp;
register unsigned int i;
unsigned int tmp_sum;

  if (rlen==0) {
    printf("\n\rOK");
  } else {
    switch (*rec_buf) {
      case 'A':
      case 'a':
        automatic();
        break;
      case 'B':
      case 'b':
        blankcheck();
        break;
      case 'C':
      case 'c':
        checksum();
        break;
      case 'D':
      case 'd':
        dump( strtoi(&rec_buf[1]) & 0xfff0 );
        break;
      case 'E':
      case 'e':
        erase();
        break;
      case 'F':
      case 'f':
        fill(&rec_buf[1]);
        break;
      case 'H':
      case 'h':
      case '?':
        help();
        break;
      case 'I':
      case 'i':
        info();
        break;
      case 'K':
      case 'k':
        lock();
        break;
      case 'L':
      case 'l':
        clear_buffer();
        load();
        break;
      case 'N':
      case 'n':
        enter(&rec_buf[1]);
        break;
      case 'P':
      case 'p':
        program();
        break;
      case 'R':
      case 'r':
        read();
        break;
      case 'S':
      case 's':
        signature();
        break;
      case 'T':
      case 't':
        type(atoi(&rec_buf[1]));
        break;
      case 'V':
      case 'v':
        verify();
        break;
	  case 'X':
	  case 'x':
	    set_data(0x0f);
		latch_enable();
		printf("\n\r%02bX",get_data());
		latch_disable();
	  break;
      default:
        printf("\n\r\n\rBad command");
      }
  }

  printf("\n\r\n\r>");

  rlen=0;
  rptr=rec_buf;
  message=0;
}


/*---------------------------------------------------------------------------*/
/*                              init_comm()                                  */
/*                                                                           */
/* Initialize serial communications                                          */
/*---------------------------------------------------------------------------*/
void init_comm(char baud)
{
  T2CON = 0x30; 	/* setup timer2 as baud rate generator */
  switch(baud) {	/* the values below are for 11.0592Mhz crystal */
    case BAUD_19200:
      RCAP2H=0xff;
      RCAP2L=0xee;
      break;
    case BAUD_9600:
      RCAP2H=0xff;
      RCAP2L=0xdc;
      break;
    case BAUD_4800:
      RCAP2H=0xff;
      RCAP2L=0xb8;
      break;
    case BAUD_2400:
      RCAP2H=0xff;
      RCAP2L=0x70;
      break;
    case BAUD_1200:
    default:
      RCAP2H=0xfe;
      RCAP2L=0xe0;
      break;
  }
  rlen=0;
  rptr=rec_buf;
  message=0;
  SCON=0x40;    /* set uart to 8-bit variable baud rate */
  baud=SBUF;
  TI=1;
  TR2=1;        /* start timer 2 */
  REN=1;        /* enable serial reception */
}


/*---------------------------------------------------------------------------*/
/*                               putchar()                                   */
/*---------------------------------------------------------------------------*/
char putchar(char c)  
{
  while (!TI);
  TI = 0;
  return (SBUF=c);
}


/*---------------------------------------------------------------------------*/
/*                                getch()                                    */
/*---------------------------------------------------------------------------*/
char getch(void)
{
char c;

  while (!RI);
  c = SBUF;
  RI = 0;
  if (c==27) return(0);
  return (c);
}


